
# q-expansion of newform 1248.2.bb.c, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1248
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[703, [-1, 0]], [1093, [-1, 0]], [833, [1, 0]], [769, [0, 1]]]
aps_data = [[0, 0], [1, 0], [-2, 2], [-1, -1], [2, -2], [-2, -3], [0, -6], [3, 3], [8, 0], [0, -2], [1, -1], [-1, -1], [8, 8], [0, -4], [8, 8], [0, -2], [4, -4], [0, 2], [1, 1], [6, -6], [-9, 9], [0, -10], [-8, -8], [10, -10], [-5, -5], [-6, 0], [-10, 0], [16, 0], [-1, 1], [-18, 0], [-8, 0], [0, 0], [0, 0], [-10, 0], [8, -8], [-1, -1], [0, -4], [9, -9], [4, 4], [-6, 0], [0, 8], [26, 0], [0, -24], [-9, 9], [2, -2], [24, 0], [4, 0], [-3, 3], [2, 2], [15, 15], [0, -14], [-10, 10], [-15, 15], [0, 24], [0, 10], [0, 0], [0, 2], [13, 13], [-22, 0], [-20, 20], [0, -6], [16, 16], [-21, 21], [16, 0], [0, 0], [2, -2], [11, 11], [0, 14], [-20, 0], [-21, -21], [24, 24], [8, 8], [0, 8], [0, -22], [-9, -9], [-8, 8], [26, 0], [-15, -15], [-6, 6], [-13, -13], [-24, 0], [-15, 15], [-12, 12], [0, -34], [-26, 0], [4, 0], [2, -2], [-19, -19], [-6, 6], [9, 9], [0, 0], [-2, -2], [15, -15], [0, 28], [3, 3], [0, 28], [-14, 14], [42, 0], [30, 0], [21, 21], [22, 0], [28, 28], [0, 28], [0, -6], [0, 14], [27, 27], [-12, -12], [14, -14], [0, -24], [16, 0], [0, 16], [-17, 17], [-8, -8], [27, -27], [-11, -11], [0, 6], [-21, -21], [-28, 0], [0, -26], [0, 0], [-3, -3], [0, -12], [0, -6], [14, -14], [21, 21], [-18, 0], [7, 7], [-36, 0], [40, 0], [15, -15], [9, -9], [6, -6], [-38, 0], [0, 10], [-12, 12], [-27, -27], [-24, 24], [13, -13], [-30, 0], [22, 0], [1, 1], [-40, 40], [-34, 0], [22, -22], [-40, 0], [30, 30], [-15, -15], [0, -2], [-4, 0], [40, -40], [15, -15], [0, -10], [0, -26], [0, 0], [0, 26], [0, 24], [0, 10], [-16, -16], [-20, 0], [28, -28], [-20, 20], [0, -6], [-33, 33], [40, 0], [-26, -26], [30, 30], [0, -40], [0, 40], [-13, 13], [46, 0], [-22, -22], [-39, -39], [40, 0], [3, 3], [34, 0], [-30, 0], [-9, 9], [24, 24], [-30, 0], [0, 50], [-25, -25], [0, 48], [0, -58], [-22, -22], [4, 4], [-14, 0], [-2, 0], [-5, -5], [-29, 29], [-20, -20], [-22, 0], [-20, -20], [22, 0], [20, 20], [0, -68], [0, 26], [-3, -3], [20, 0], [-2, 2], [0, 24], [-18, -18], [0, 64], [-21, 21], [-30, 0], [-8, 8], [0, 30], [-1, 1], [16, 0], [-2, -2], [0, 50], [0, 2], [0, 50], [0, 30], [10, -10], [-16, 16], [-21, 21], [0, -22], [-14, 0], [10, -10], [44, 44], [0, 36], [9, 9], [-22, -22], [-25, 25], [0, -72], [22, 0], [54, 0], [0, 12], [32, 0], [4, -4], [-2, 0], [62, 0], [-7, 7], [0, -42], [26, 0], [-26, 26], [-23, 23], [-16, -16], [0, -28], [0, -40], [36, 36], [0, 34], [0, -66], [5, -5], [14, 14], [4, 0], [-15, -15], [-48, 48], [3, 3], [4, 0], [9, 9], [-12, -12], [22, 22], [0, 10], [0, 30], [6, -6], [0, -10], [29, 29], [-30, 0], [11, 11], [32, 0], [-36, 0], [-39, 39], [0, 16], [2, -2], [-36, 0], [42, -42], [-36, -36], [-11, 11], [-74, 0], [-4, 0], [-55, -55], [23, -23], [62, 0], [4, 0], [-15, 15], [-2, -2], [-37, -37], [7, -7], [0, -48], [0, -56], [-9, -9], [0, 44], [-7, 7], [-15, 15], [-72, 0], [16, 0], [-16, 16], [-27, -27], [0, 18], [0, -54], [40, 0], [22, 22], [4, -4], [0, 28], [18, 0], [0, 40], [-30, 0], [64, 0], [37, -37], [0, -8], [14, 14], [-50, 0]]
